'use strict';

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _underscore = require('underscore');

var _underscore2 = _interopRequireDefault(_underscore);

var _gulpUtil = require('gulp-util');

var _gulpUtil2 = _interopRequireDefault(_gulpUtil);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Elixir is a wrapper around Gulp.
 *
 * @param {Function} recipe
 */
var Elixir = function Elixir(recipe) {
    // Perform any last-minute initializations.
    init();

    // Load all of Elixir's task definitions.
    require('require-dir')('./tasks');

    // Load the user's Gulpfile recipe.
    recipe(Elixir.mixins);

    // And run their chosen tasks.
    Elixir.tasks.forEach(function (task) {
        return task.toGulp();
    });
};

Elixir.mixins = {};
Elixir.isWatching = function () {
    return _gulpUtil2.default.env._.indexOf('watch') > -1;
};
Elixir.Log = require('./Logger').default;
Elixir.GulpPaths = require('./GulpPaths').default;
Elixir.config = require('./Config').default;
Elixir.Plugins = require('gulp-load-plugins')();
Elixir.Task = require('./Task').default(Elixir);
Elixir.tasks = new (require('./TaskCollection').default)();

Elixir.hooks = { before: [], watch: [] };
Elixir.onWatch = function (func) {
    return Elixir.hooks.watch.push(func);
};
Elixir.before = function (func) {
    return Elixir.hooks.before.push(func);
};

/**
 * Register a new task with Elixir.
 *
 * @param {string}   name
 * @param {Function} callback
 */
Elixir.extend = function (name, callback) {
    Elixir.mixins[name] = function () {
        callback.apply(this, arguments);

        return this.mixins;
    }.bind(this);
};

/**
 * Allow for config overrides, via an elixir.json file.
 *
 * @param {string} file
 */
Elixir.setDefaultsFrom = function (file) {
    var overrides = void 0;

    if (_fs2.default.existsSync(file)) {
        overrides = JSON.parse(_fs2.default.readFileSync(file, 'utf8'));

        _underscore2.default.mixin({
            deepExtend: require('underscore-deep-extend')(_underscore2.default)
        });

        _underscore2.default.deepExtend(Elixir.config, overrides);
    }
}('elixir.json');

/**
 * Perform any last-minute initializations.
 */
function init() {
    if (!Elixir.config.notifications) {
        process.env.DISABLE_NOTIFIER = true;
    }

    Elixir.Notification = require('./Notification').default;
};

module.exports = Elixir;